#ifndef ICOSAHEDRON_H
#define ICOSAHEDRON_H

#include "R3Graph.h"

const double GOLD_SECT = 1.6180339887498948482;

class Icosahedron {
public:
    typedef int Face[3];
    typedef int Edge[2];

    // Triangle edges are defined as follows:
    //     sign * (edge_index+1), sign = +-1, 
    //     sign is negative in case of inverse orientation.
    typedef int Triangle[3]; 

    // Vertices of icosahedron
    int numVertices;
    R3Graph::R3Point* vertices;

    int numFaces;
    Face* faces;
    Triangle* triangles;

    int numEdges;
    Edge* edges;

public:
    Icosahedron();
    ~Icosahedron();
    void initialize();
    void split();
    R3Graph::R3Point center() const {
        return R3Graph::R3Point(0., 0., 0.);
    }
    R3Graph::R3Vector direction(int vertexIdx) const;
    double radius() const { return 1.; }
};

#endif
