//
// Infinite number of space
// decompostions into equal tetrahedra exists?
//
#include <cstdio>
#include "R3Graph.h"
#include "tetrahed.h"
using namespace R3Graph;

static const double EPS = 0.0001;

int main() {
    double dz = sqrt(3.) / 2.;
    double dx = 1.;
    double dy0 = 1./3., dy1 = 1.5;
    double dy;
    double h0 = 0., h1 = 1.;
    double dh = 0.01;
    double deltay = 0.00001;
    double asp0, asp1, asp2;

    //
    //        p4
    //  p5
    //              p2
    //        p3
    //  p0
    //              p1
    //
    R3Point p1(dx, 0., 0.);

    double h = h0;
    while (h <= h1) {
        R3Point p0(0., h, 0.);
        double minAspDiff = 1e+10;
        double minAsp0, minAsp2;
        double minDy;
        dy = dy0;
        while (dy <= dy1) {
            R3Point p2(dx, dy, 0.);
            R3Point p3(dx/2., (p0.y + p2.y)/2., dz);
            R3Point p4(p3.x, p3.y + dy, dz);
            R3Point p5(0., p0.y + dy, 0.);

            Tetrahedron t0(p0, p1, p2, p3);
            Tetrahedron t1(p0, p2, p5, p3);
            Tetrahedron t2(p3, p4, p5, p2);

            asp0 = t0.aspect();
            asp1 = t1.aspect();
            asp2 = t2.aspect();

            double diff = fabs(asp0 - asp2);
            if (diff < minAspDiff) {
                minAspDiff = diff;
                minAsp0 = asp0;
                minAsp2 = asp2;
                minDy = dy;
            }

            /*...
            if (fabs(asp0 - asp2) <= EPS) {
                printf(
                    "h=%lf dy=%lf asp0=%lf asp2=%lf\n",
                    h, dy, asp0, asp2
                );
            }
            ...*/

            dy += deltay;
        } // end while (dy <= dy1)

        printf(
            "h=%lf dy=%lf asp0=%lf asp2=%lf\n",
            h, minDy, minAsp0, minAsp2
        );
        if (minAspDiff <= EPS)
            printf("Found! h=%lf, dy=%lf\n", h, minDy);

        h += dh;
    } // end while (h <= h1)

    printf("\n======================\n");

    // Optimal tri-prism
    printf("\n======================\n");
    h = 1./3.; dy = 1.;
    R3Point p0(0., h, 0.);
    R3Point p2(dx, dy, 0.);
    R3Point p3(dx/2., (p0.y + p2.y)/2., dz);
    R3Point p4(p3.x, p3.y + dy, dz);
    R3Point p5(0., p0.y + dy, 0.);

    Tetrahedron t0(p0, p1, p2, p3);
    Tetrahedron t1(p0, p2, p5, p3);
    Tetrahedron t2(p3, p4, p5, p2);

    asp0 = t0.aspect();
    asp1 = t1.aspect();
    asp2 = t2.aspect();

    double meanAsp = (asp0 + asp1 + asp2) / 3.;
    printf("Optimal tri-prism decomposition:\n");
    printf(
        "h=%lf asp0=%lf asp1=%lf asp2=%lf  mean asp=%lf\n",
        h, asp0, asp1, asp2, meanAsp
    );

    printf("Tetrahedron 0:\n");
    printf("  Vertices:\n");
    printf("    (%lf, %lf, %lf)\n", t0.vertex[0].x, t0.vertex[0].y, t0.vertex[0].z);
    printf("    (%lf, %lf, %lf)\n", t0.vertex[1].x, t0.vertex[1].y, t0.vertex[0].z);
    printf("    (%lf, %lf, %lf)\n", t0.vertex[2].x, t0.vertex[2].y, t0.vertex[0].z);
    printf("    (%lf, %lf, %lf)\n", t0.vertex[3].x, t0.vertex[3].y, t0.vertex[3].z);
    printf("  Edges:\n");
    printf("    %lf, %lf, %lf\n    %lf, %lf, %lf\n",
        t0.vertex[0].distance(t0.vertex[1]),
        t0.vertex[0].distance(t0.vertex[2]),
        t0.vertex[0].distance(t0.vertex[3]),
        t0.vertex[1].distance(t0.vertex[2]),
        t0.vertex[1].distance(t0.vertex[3]),
        t0.vertex[2].distance(t0.vertex[3])
    );

    printf("Tetrahedron 1:\n");
    printf("  Vertices:\n");
    printf("    (%lf, %lf, %lf)\n", t1.vertex[0].x, t1.vertex[0].y, t1.vertex[0].z);
    printf("    (%lf, %lf, %lf)\n", t1.vertex[1].x, t1.vertex[1].y, t1.vertex[0].z);
    printf("    (%lf, %lf, %lf)\n", t1.vertex[2].x, t1.vertex[2].y, t1.vertex[0].z);
    printf("    (%lf, %lf, %lf)\n", t1.vertex[3].x, t1.vertex[3].y, t1.vertex[3].z);
    printf("  Edges:\n");
    printf("    %lf, %lf, %lf\n    %lf, %lf, %lf\n",
        t1.vertex[0].distance(t1.vertex[1]),
        t1.vertex[0].distance(t1.vertex[2]),
        t1.vertex[0].distance(t1.vertex[3]),
        t1.vertex[1].distance(t1.vertex[2]),
        t1.vertex[1].distance(t1.vertex[3]),
        t1.vertex[2].distance(t1.vertex[3])
    );

    printf("Tetrahedron 2:\n");
    printf("  Vertices:\n");
    printf("    (%lf, %lf, %lf)\n", t2.vertex[0].x, t2.vertex[0].y, t2.vertex[0].z);
    printf("    (%lf, %lf, %lf)\n", t2.vertex[1].x, t2.vertex[1].y, t2.vertex[0].z);
    printf("    (%lf, %lf, %lf)\n", t2.vertex[2].x, t2.vertex[2].y, t2.vertex[0].z);
    printf("    (%lf, %lf, %lf)\n", t2.vertex[3].x, t2.vertex[3].y, t2.vertex[3].z);
    printf("  Edges:\n");
    printf("    %lf, %lf, %lf\n    %lf, %lf, %lf\n",
        t2.vertex[0].distance(t2.vertex[1]),
        t2.vertex[0].distance(t2.vertex[2]),
        t2.vertex[0].distance(t2.vertex[3]),
        t2.vertex[1].distance(t2.vertex[2]),
        t2.vertex[1].distance(t2.vertex[3]),
        t2.vertex[2].distance(t2.vertex[3])
    );

    return 0;
}
