#ifndef TETRAHEDRON_H
#define TETRAHEDRON_H

#include "R3Graph.h"

namespace R3Graph {

class Tetrahedron {
public:
    R3Point vertex[4];

    Tetrahedron() {}

    Tetrahedron(const Tetrahedron& t) {
        vertex[0] = t.vertex[0];
        vertex[1] = t.vertex[1];
        vertex[2] = t.vertex[2];
        vertex[3] = t.vertex[3];
    }

    Tetrahedron(
        const R3Point& p0,
        const R3Point& p1,
        const R3Point& p2,
        const R3Point& p3
    ) {
        vertex[0] = p0;
        vertex[1] = p1;
        vertex[2] = p2;
        vertex[3] = p3;
    }

    Tetrahedron operator=(const Tetrahedron& t) {
        vertex[0] = t.vertex[0];
        vertex[1] = t.vertex[1];
        vertex[2] = t.vertex[2];
        vertex[3] = t.vertex[3];
        return *this;
    }

    double volume() const {
        return R3Vector::volume(
            vertex[1] - vertex[0],
            vertex[2] - vertex[0],
            vertex[3] - vertex[0]
        ) / 6.;
    }

    double innerRadius() const;
    double outerRadius() const;
    double aspect() const;
};

} // end of namespace R3Graph

#endif
